@@ -10,11 +10,11 @@ O.engines=[
 	NewtonIntegrator(damping = 0.01,gravity=[0,0,-9.81])
 ]
 
-from yade import utils
 
-O.bodies.append(utils.sphere([0,0,6],1,fixed=False, color=[0,1,0]))
-O.bodies.append(utils.sphere([0,0,0],1,fixed=True, color=[0,0,1]))
-O.dt=.2*utils.PWaveTimeStep()
+
+O.bodies.append(sphere([0,0,6],1,fixed=False, color=[0,1,0]))
+O.bodies.append(sphere([0,0,0],1,fixed=True, color=[0,0,1]))
+O.dt=.2*PWaveTimeStep()
 
 from yade import qt
 qt.Controller()
