@@ -407,8 +407,8 @@ void GLViewer::centerScene(){
 			min=Vector3r(inf,inf,inf); max=Vector3r(-inf,-inf,-inf);
 			FOREACH(const shared_ptr<Body>& b, *rb->bodies){
 				if(!b) continue;
-				max=max.cwise().max(b->state->pos);
-				min=min.cwise().min(b->state->pos);
+				max=max.cwiseMax(b->state->pos);
+				min=min.cwiseMin(b->state->pos);
 			}
 			if(isinf(min[0])||isinf(min[1])||isinf(min[2])||isinf(max[0])||isinf(max[1])||isinf(max[2])){ LOG_DEBUG("No min/max computed from bodies either, setting cube (-1,-1,-1)×(1,1,1)"); min=-Vector3r::Ones(); max=Vector3r::Ones(); }
 		} else {LOG_DEBUG("Using scene's Aabb");}
