@@ -26,7 +26,7 @@ namespace CGT{
 		void Compute_Permeability();
 		void GaussSeidel(Real dt=0);
 		void DisplayStatistics();
-		void ComputeEdgesSurfaces();
+		void computeEdgesSurfaces();
 		double boundaryFlux(unsigned int boundaryId);
 		#ifdef EIGENSPARSE_LIB
 		//Eigen's sparse matrix for forces computation
