@@ -5,6 +5,6 @@ YADE_PLUGIN((ForceResetter));
 
 void ForceResetter::action(){
 	scene->forces.reset(scene->iter);
-	if(unlikely(scene->trackEnergy)) scene->energy->resetResettables();
+	if(scene->trackEnergy) scene->energy->resetResettables();
 }
 
