@@ -1,6 +1,7 @@
 /*************************************************************************
-*  Copyright (C) 2009 by Emanuele Catalano                               *
-*  emanuele.catalano@hmg.inpg.fr                                            *
+*  Copyright (C) 2009 by Emanuele Catalano <catalano@grenoble-inp.fr>    *
+*  Copyright (C) 2009 by Bruno Chareyre <bruno.chareyre@hmg.inpg.fr>     *
+*  Copyright (C) 2012 by Donia Marzougui <donia.marzougui@grenoble-inp.fr>*
 *                                                                        *
 *  This program is free software; it is licensed under the terms of the  *
 *  GNU General Public License v2 or later. See file LICENSE for details. *
@@ -33,6 +34,7 @@ class FlowEngine : public PartialEngine
 	typedef FlowSolver::Finite_cells_iterator				Finite_cells_iterator;
 	typedef FlowSolver::Cell_handle						Cell_handle;
 	typedef RTriangulation::Finite_edges_iterator				Finite_edges_iterator;
+	typedef FlowSolver::Vertex_handle                    			Vertex_handle;
 
 
 	
@@ -42,8 +44,10 @@ class FlowEngine : public PartialEngine
 		volatile bool backgroundCompleted;
 		Cell cachedCell;
 		struct posData {Body::id_t id; Vector3r pos; Real radius; bool isSphere; bool exists; posData(){exists=0;}};
-		vector<posData> positionBuffer;
-		void setPositionsBuffer();
+		vector<posData> positionBufferCurrent;//reflect last known positions before we start computations
+		vector<posData> positionBufferParallel;//keep the positions from a given step for multithread factorization
+		//copy positions in a buffer for faster and/or parallel access
+		void setPositionsBuffer(bool current);
 
 	public :
 		int retriangulationLastIter;
@@ -53,6 +57,7 @@ class FlowEngine : public PartialEngine
 		int id_offset;
 		double Eps_Vol_Cumulative;
 		int ReTrg;
+		int ellapsedIter;
 		TPL void initSolver (Solver& flow);
 		TPL void Triangulate (Solver& flow);
 		TPL void AddBoundary (Solver& flow);
@@ -71,7 +76,7 @@ class FlowEngine : public PartialEngine
 		TPL void setImposedPressure(unsigned int cond, Real p,Solver& flow);
 		TPL void clearImposedPressure(Solver& flow);
 		TPL void clearImposedFlux(Solver& flow);
-		TPL void ApplyViscousForces(Solver& flow);
+		TPL void ComputeViscousForces(Solver& flow);
 		TPL Real getCellFlux(unsigned int cond, const shared_ptr<Solver>& flow);
 		TPL Real getBoundaryFlux(unsigned int boundary,Solver& flow) {return flow->boundaryFlux(boundary);}
 		TPL Vector3r fluidForce(unsigned int id_sph, Solver& flow) {
@@ -86,6 +91,21 @@ class FlowEngine : public PartialEngine
 			return (flow->viscousBodyStress.size()>id_sph)?flow->viscousBodyStress[id_sph]:Matrix3r::Zero();}
 		TPL Matrix3r bodyNormalLubStress(unsigned int id_sph, Solver& flow) {
 			return (flow->lubBodyStress.size()>id_sph)?flow->lubBodyStress[id_sph]:Matrix3r::Zero();}
+		TPL python::list getConstrictions(bool all, Solver& flow) {
+			vector<Real> csd=flow->getConstrictions(); python::list pycsd;
+			for (unsigned int k=0;k<csd.size();k++) if ((all && csd[k]!=0) || csd[k]>0) pycsd.append(csd[k]); return pycsd;}
+		TPL python::list getConstrictionsFull(bool all, Solver& flow) {
+			vector<Constriction> csd=flow->getConstrictionsFull(); python::list pycsd;
+			for (unsigned int k=0;k<csd.size();k++) if ((all && csd[k].second[0]!=0) || csd[k].second[0]>0) {
+				python::list cons;
+				cons.append(csd[k].first.first);
+				cons.append(csd[k].first.second);
+				cons.append(csd[k].second[0]);
+				cons.append(csd[k].second[1]);
+				cons.append(csd[k].second[2]);
+				cons.append(csd[k].second[3]);
+				pycsd.append(cons);}
+			return pycsd;}
 		
 		template<class Cellhandle>
 		Real Volume_cell_single_fictious (Cellhandle cell);
@@ -99,19 +119,19 @@ class FlowEngine : public PartialEngine
 		void Average_real_cell_velocity();
 		void saveVtk() {solver->saveVtk();}
 		vector<Real> AvFlVelOnSph(unsigned int id_sph) {return solver->Average_Fluid_Velocity_On_Sphere(id_sph);}
-		python::list getConstrictions() {
-			vector<Real> csd=solver->getConstrictions(); python::list pycsd;
-			for (unsigned int k=0;k<csd.size();k++) pycsd.append(csd[k]); return pycsd;}
+
 		void setBoundaryVel(Vector3r vel) {topBoundaryVelocity=vel; Update_Triangulation=true;}
 		void PressureProfile(double wallUpY, double wallDownY) {return solver->MeasurePressureProfile(wallUpY,wallDownY);}
 		double MeasurePorePressure(Vector3r pos){return solver->MeasurePorePressure(pos[0], pos[1], pos[2]);}
 		TPL int getCell(double posX, double posY, double posZ, Solver& flow){return flow->getCell(posX, posY, posZ);}
 		double MeasureAveragedPressure(double posY){return solver->MeasureAveragedPressure(posY);}
 		double MeasureTotalAveragedPressure(){return solver->MeasureTotalAveragedPressure();}
+		#ifdef LINSOLV
 		TPL void exportMatrix(string filename,Solver& flow) {if (useSolver==3) flow->exportMatrix(filename.c_str());
 			else cerr<<"available for Cholmod solver (useSolver==3)"<<endl;}
 		TPL void exportTriplets(string filename,Solver& flow) {if (useSolver==3) flow->exportTriplets(filename.c_str());
 			else cerr<<"available for Cholmod solver (useSolver==3)"<<endl;}
+		#endif
 
 		void emulateAction(){
 			scene = Omega::instance().getScene().get();
@@ -132,9 +152,13 @@ class FlowEngine : public PartialEngine
 		Real 		_getCellFlux(unsigned int cond) {return getCellFlux(cond,solver);}
 		Real 		_getBoundaryFlux(unsigned int boundary) {return getBoundaryFlux(boundary,solver);}
 		int		_getCell(Vector3r pos) {return getCell(pos[0],pos[1],pos[2],solver);}
+		#ifdef LINSOLV
 		void 		_exportMatrix(string filename) {exportMatrix(filename,solver);}
 		void 		_exportTriplets(string filename) {exportTriplets(filename,solver);}
-		
+		#endif
+		python::list 	_getConstrictions(bool all) {return getConstrictions(all,solver);}
+		python::list 	_getConstrictionsFull(bool all) {return getConstrictionsFull(all,solver);}
+
 		virtual ~FlowEngine();
 
 		virtual void action();
@@ -173,6 +197,7 @@ class FlowEngine : public PartialEngine
 					((Real,maxKdivKmean,100,,"define the max K value (see :yref:`FlowEngine::clampKValues`)"))
 					((double,permeability_factor,0.0,,"permability multiplier"))
 					((double,viscosity,1.0,,"viscosity of fluid"))
+					((double,stiffness, 10000,,"stiffness modulus"))
 					((Real,loadFactor,1.1,,"Load multiplicator for oedometer test"))
 					((double, K, 0,, "Permeability of the sample"))
 					((int, useSolver, 0,, "Solver to use 0=G-Seidel, 1=Taucs, 2-Pardiso, 3-CHOLMOD"))
@@ -208,8 +233,11 @@ class FlowEngine : public PartialEngine
 					((bool, viscousShear, false,,"Compute viscous shear terms as developped by Donia Marzougui"))
 					((bool, shearLubrication, false,,"Compute shear lubrication force as developped by Brule"))
 					((double, eps, 0.00001,,"minimum distance between particles"))
+					((bool, pressureForce, true,,"Compute the pressure field and associated fluid forces. WARNING: turning off means fluid flow is not computed at all."))
 
 					((bool, normalLubrication, false,,"Compute normal lubrication force as developped by Brule"))
+					((bool, viscousNormalBodyStress, false,,"Compute normal viscous stress applied on each body"))
+					((bool, viscousShearBodyStress, false,,"Compute shear viscous stress applied on each body"))
 					((bool, multithread, false,,"Build triangulation and factorize in the background (multi-thread mode)"))
 					#ifdef EIGENSPARSE_LIB
 					((int, numSolveThreads, 1,,"number of openblas threads in the solve phase."))
@@ -231,6 +259,7 @@ class FlowEngine : public PartialEngine
 					eps_vol_max=Eps_Vol_Cumulative=retriangulationLastIter=0;
 					ReTrg=1;
 					backgroundCompleted=true;
+					ellapsedIter=0;
 					,
 					.def("imposeFlux",&FlowEngine::_imposeFlux,(python::arg("pos"),python::arg("p")),"Impose incoming flux in boundary cell of location 'pos'.")
 					.def("imposePressure",&FlowEngine::_imposePressure,(python::arg("pos"),python::arg("p")),"Impose pressure in cell of location 'pos'. The index of the condition is returned (for multiple imposed pressures at different points).")
@@ -239,7 +268,8 @@ class FlowEngine : public PartialEngine
 					.def("clearImposedFlux",&FlowEngine::_clearImposedFlux,"Clear the list of points with flux imposed.")
 					.def("getCellFlux",&FlowEngine::_getCellFlux,(python::arg("cond")),"Get influx in cell associated to an imposed P (indexed using 'cond').")
 					.def("getBoundaryFlux",&FlowEngine::_getBoundaryFlux,(python::arg("boundary")),"Get total flux through boundary defined by its body id.")
-					.def("getConstrictions",&FlowEngine::getConstrictions,"Get the list of constrictions (inscribed circle) for all finite facets.")
+					.def("getConstrictions",&FlowEngine::_getConstrictions,(python::arg("all")=true),"Get the list of constriction radii (inscribed circle) for all finite facets (if all==True) or all facets not incident to a virtual bounding sphere (if all==False).  When all facets are returned, negative radii denote facet incident to one or more fictious spheres.")
+					.def("getConstrictionsFull",&FlowEngine::_getConstrictionsFull,(python::arg("all")=true),"Get the list of constrictions (inscribed circle) for all finite facets (if all==True), or all facets not incident to a fictious bounding sphere (if all==False). When all facets are returned, negative radii denote facet incident to one or more fictious spheres. The constrictions are returned in the format {{cell1,cell2}{rad,nx,ny,nz}}")
 					.def("saveVtk",&FlowEngine::saveVtk,"Save pressure field in vtk format.")
 					.def("AvFlVelOnSph",&FlowEngine::AvFlVelOnSph,(python::arg("Id_sph")),"Compute a sphere-centered average fluid velocity")
 					.def("fluidForce",&FlowEngine::_fluidForce,(python::arg("Id_sph")),"Return the fluid force on sphere Id_sph.")
@@ -257,8 +287,10 @@ class FlowEngine : public PartialEngine
 					.def("updateBCs",&FlowEngine::_updateBCs,"tells the engine to update it's boundary conditions before running (especially useful when changing boundary pressure - should not be needed for point-wise imposed pressure)")
 					.def("emulateAction",&FlowEngine::emulateAction,"get scene and run action (may be used to manipulate engine outside the main loop).")
 					.def("getCell",&FlowEngine::_getCell,(python::arg("pos")),"get id of the cell containing (X,Y,Z).")
+					#ifdef LINSOLV
 					.def("exportMatrix",&FlowEngine::_exportMatrix,(python::arg("filename")="matrix"),"Export system matrix to a file with all entries (even zeros will displayed).")
 					.def("exportTriplets",&FlowEngine::_exportTriplets,(python::arg("filename")="triplets"),"Export system matrix to a file with only non-zero entries.")
+					#endif
 					)
 		DECLARE_LOGGER;
 };
@@ -311,19 +343,6 @@ class PeriodicFlowEngine : public FlowEngine
 		inline void locateCell(Cell_handle baseCell, unsigned int& index, int& baseIndex, shared_ptr<FlowSolver>& flow, unsigned int count=0);
 		Vector3r meanVelocity();
 
-		python::list getConstrictionsFull() {
-			vector<Constriction> csd=solver->getConstrictionsFull(); python::list pycsd;
-			for (unsigned int k=0;k<csd.size();k++) {
-				python::list cons;
-				cons.append(csd[k].first.first);
-				cons.append(csd[k].first.second);
-				cons.append(csd[k].second[0]);
-				cons.append(csd[k].second[1]);
-				cons.append(csd[k].second[2]);
-				cons.append(csd[k].second[3]);
-				pycsd.append(cons);}
-			return pycsd;}
-		
 		virtual ~PeriodicFlowEngine();
 
 		virtual void action();
@@ -339,7 +358,6 @@ class PeriodicFlowEngine : public FlowEngine
 		Matrix3r 	_bodyShearLubStress(unsigned int id_sph) {return bodyShearLubStress(id_sph,solver);}
 		Matrix3r 	_bodyNormalLubStress(unsigned int id_sph) {return bodyNormalLubStress(id_sph,solver);}
 
-// 		void 		saveVtk() {solver->saveVtk();} // FIXME: need to adapt vtk recorder to periodic case
 		Vector3r 	_fluidForce(unsigned int id_sph) {return fluidForce(id_sph, solver);}
 		void 		_imposeFlux(Vector3r pos, Real flux) {return imposeFlux(pos,flux,*solver);}
 		unsigned int 	_imposePressure(Vector3r pos, Real p) {return imposePressure(pos,p,this->solver);}
@@ -351,12 +369,26 @@ class PeriodicFlowEngine : public FlowEngine
 		void 		PressureProfile(double wallUpY, double wallDownY) {return solver->MeasurePressureProfile(wallUpY,wallDownY);}
 
 		int		_getCell(Vector3r pos) {return getCell(pos[0],pos[1],pos[2],solver);}
+		#ifdef LINSOLV
 		void 		_exportMatrix(string filename) {exportMatrix(filename,solver);}
 		void 		_exportTriplets(string filename) {exportTriplets(filename,solver);}
+		#endif
 		
 // 		void 		_setImposedPressure(unsigned int cond, Real p) {setImposedPressure(cond,p,solver);}
 // 		void 		_clearImposedPressure() {clearImposedPressure(solver);}
 		Real 		_getCellFlux(unsigned int cond) {return getCellFlux(cond,solver);}
+		python::list 	_getConstrictions(bool all) {return getConstrictions(all,solver);}
+		python::list 	_getConstrictionsFull(bool all) {return getConstrictionsFull(all,solver);}
+
+		//commodities
+		void compTessVolumes() {
+			solver->T[solver->currentTes].Compute();
+			solver->T[solver->currentTes].ComputeVolumes();
+		}
+		Real getVolume (Body::id_t id) {
+			if (solver->T[solver->currentTes].Max_id() <= 0) {emulateAction(); LOG_WARN("Not triangulated yet, emulating action");}
+			if (solver->T[solver->currentTes].Volume(id) == -1) {compTessVolumes(); LOG_WARN("Computing all volumes now, as you did not request it explicitely.");}
+			return (solver->T[solver->currentTes].Max_id() >= id) ? solver->T[solver->currentTes].Volume(id) : -1;}
 
 		YADE_CLASS_BASE_DOC_ATTRS_INIT_CTOR_PY(PeriodicFlowEngine,FlowEngine,"An engine to solve flow problem in saturated granular media",
 			((Real,duplicateThreshold, 0.06,,"distance from cell borders that will triger periodic duplication in the triangulation |yupdate|"))
@@ -388,9 +420,14 @@ class PeriodicFlowEngine : public FlowEngine
 			.def("updateBCs",&PeriodicFlowEngine::_updateBCs,"tells the engine to update it's boundary conditions before running (especially useful when changing boundary pressure - should not be needed for point-wise imposed pressure)")
 			
 			.def("getCell",&PeriodicFlowEngine::_getCell,python::arg("pos"),"get id of the cell containing 'pos'.")
-			.def("getConstrictionsFull",&PeriodicFlowEngine::getConstrictionsFull,"Get the list of constrictions (inscribed circle) for all finite facets.")
+			.def("getConstrictions",&PeriodicFlowEngine::_getConstrictions,(python::arg("all")=true),"Get the list of constriction radii (inscribed circle) for all finite facets (if all==True) or all facets not incident to a virtual bounding sphere (if all==False).  When all facets are returned, negative radii denote facet incident to one or more fictious spheres.")
+			.def("getConstrictionsFull",&PeriodicFlowEngine::_getConstrictionsFull,(python::arg("all")=true),"Get the list of constrictions (inscribed circle) for all finite facets (if all==True), or all facets not incident to a fictious bounding sphere (if all==False). When all facets are returned, negative radii denote facet incident to one or more fictious spheres. The constrictions are returned in the format {{cell1,cell2}{rad,nx,ny,nz}}")
+			#ifdef LINSOLV
 			.def("exportMatrix",&PeriodicFlowEngine::_exportMatrix,(python::arg("filename")="matrix"),"Export system matrix to a file with all entries (even zeros will displayed).")
 			.def("exportTriplets",&PeriodicFlowEngine::_exportTriplets,(python::arg("filename")="triplets"),"Export system matrix to a file with only non-zero entries.")
+			#endif
+			.def("compTessVolumes",&PeriodicFlowEngine::compTessVolumes,"Like TesselationWrapper::computeVolumes()")
+			.def("volume",&PeriodicFlowEngine::getVolume,(python::arg("id")=0),"Returns the volume of Voronoi's cell of a sphere.")
 		)
 		DECLARE_LOGGER;
 
